#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "define.h"
#include "dxc9000.h"
#include "serial.h"

char *shutterString[]={
  "16.0sec","15.0sec","14.0sec","13.0sec","12.0sec","11.0sec","10.0sec",
  "9.0sec","8.0sec","7.0sec","6.0sec","5.0sec","4.0sec","3.5sec","3.0sec",
  "2.5sec","2.0sec","1.5sec","1.0sec","0.5sec","0.3sec","0.2sec","0.1sec",
  "1/60","FL","1/125","1/250","1/500","1/1000","1/2000","1/4000","1/10000"};

char *commandString[]={
  "ZOOM",
  "FOCUS",
  "IRIS-MODE",
  "IRIS",
  "C.TEMP",
  "GAMMA",
  "GAIN",
  "GAIN-STEP",
  "DETAIL",
  "DETAIL-LEVEL",
  "SHUTTER",
  "SHUTTERSPEED",
  "WHITEBALLANCE",
  "R.GAIN",
  "B.GAIN",
  "SCANMODE",
  "EXT.CTRL",
  "H-PHASE",
  "SC-PHASE-Rough",
  "SC-PHASE-Fine",
  "G.SYNC",
  "D-SUB-VIDEO",
  "FREEZE",
  "AE-DETECTION",
  "EXT.TRIGGER",
  "FREEZE-MODE",
  "PROTECT",
  "TRIG.PULSE",
  "COLORBARS" };

/* ============================================================ */
/* ޥ(֤ͤΥåĤ) */
void Dxc9000SendCommand(int aux, char *command){
  int i;
  char reply[5];

  if (OPT_DEBUG == OPT_TRUE) return;

  for (i=0; i<4; i++){ SerialPutc(aux,command[i]); usleep(3000); }
  SerialPutc(aux,0x0D); usleep(3000);

  SerialGetString(aux, reply, 5);
  if (strncmp("OKAY",reply,4)==0) return;
  if (strncmp("CERR",reply,4)==0){ fprintf(stderr,"Error: CERR\n"); exit(1);}
  else { fprintf(stderr,"Error:(Unknown)\n"); exit(1); }
}

/* ============================================================ */
/* DXC9000ȥ⡼ɤؤ */
void Dxc9000SetUp(int aux){
  int i;
  char reply[5];
  char command[] = "VERS";

  /* 餫V3.nμ */
  while(1){
    for (i=0; i<4; i++){ SerialPutc(aux,command[i]); usleep(3000);}
    SerialPutc(aux,0x0D);
    usleep(3000);  
    SerialGetString(aux, reply, 5);
    if (strncmp("ACTV",reply,4)==0) continue;
    if (strncmp("OKAY",reply,4)==0) continue;
    break;
  }

  /* ΤV3.n ɤγǧ */
  if (strncmp("CERR",reply,4)==0){
    fprintf(stderr,"Error: (CERR)\n"); exit(1);
  }

  fprintf(stderr, "ROM version: %s",reply);
  if (strncmp("V3.",reply,3)!=0){
    fprintf(stderr,"Error: This camera is NOT DXC-9000!\n"); exit(1);
  }

}
/* ============================================================ */
void Dxc9000State(int aux, int D, unsigned char value, unsigned char mask){
  char data[5];
  if (D==1) {
    sprintf(data,"D1%02X",value);
    Dxc9000SendCommand(aux,data);
    sprintf(data,"DA%02X",mask);
    Dxc9000SendCommand(aux,data);
  } else if (D==2) {
    sprintf(data,"D2%02X",value);
    Dxc9000SendCommand(aux,data);
    sprintf(data,"DB%02X",mask);
    Dxc9000SendCommand(aux,data);
  } else if (D==3) {
    sprintf(data,"D3%02X",value);
    Dxc9000SendCommand(aux,data);
    sprintf(data,"DC%02X",mask);
    Dxc9000SendCommand(aux,data);
  }
}

/* ============================================================ */
/* IRIS (0255) */
void Dxc9000Iris(int aux,int value){
  char data[5];
  
  if (value < 0|| value > 255 ){
    fprintf(stderr,"Error: Iris is out of range.\n"); exit(1);
  }
  sprintf(data,"AE%02X",value);
  Dxc9000SendCommand(aux,data);
  Camera.iris = value;
}

/* ============================================================ */
/* ZOOM (0255) */
void Dxc9000Zoom(int aux,int value){
  char data[5];
  
  if (value < 0 || value > 255 ){
    fprintf(stderr,"Error: Zoom is out of range.\n"); exit(1);
  }
  sprintf(data,"85%02X",value);
  Dxc9000SendCommand(aux,data);
  Camera.zoom = value;
}

/* ============================================================ */
/* FOCUS (0255) */
void Dxc9000Focus(int aux,int value){
  char data[5];
  
  if (value < 0 || value > 255 ){
    fprintf(stderr,"Error: Focus is out of range.\n"); exit(1);
  }
  sprintf(data,"84%02X",value);
  Dxc9000SendCommand(aux,data);
  Camera.focus = value;
}

/* ============================================================ */
/* ٤ (DXC9000_CTEMP_3200K 뤤 DXC9000_CTEMP_5600K) */
void Dxc9000ColorTemperture(int aux,int value){

  if (value == DXC9000_CTEMP_3200K ){
    Dxc9000SendCommand(aux,"D100");
    Camera.D1 &= 0x7F;
  } else if (value == DXC9000_CTEMP_5600K ){
    Dxc9000SendCommand(aux,"D180");
    Camera.D1 |= 0x80;
  } else {
    fprintf(stderr,"Error: Color Temperture.\n"); exit(1);
  }
  Dxc9000SendCommand(aux,"DA80");
}

/* ============================================================ */
/* ̵ͭ (DXC9000_OFF 뤤 DXC9000_ON) */
void Dxc9000Gamma(int aux, int value){
  if (value == DXC9000_OFF){
    Dxc9000SendCommand(aux,"D100");
    Camera.D1 &= 0xBF;
  } else if (value == DXC9000_ON){
    Dxc9000SendCommand(aux,"D140");
    Camera.D1 |= 0x40;
  } else {
    fprintf(stderr,"Error: Gamma.\n"); exit(1);
  }
  Dxc9000SendCommand(aux,"DA40");
}

/* ============================================================ */
/* IRIS MODE (DXC9000_IRISMODE_AUTO 뤤 DXC9000_IRISMODE_FIX) */
void Dxc9000IrisMode(int aux, int value){
  if (value == DXC9000_IRISMODE_AUTO){
    Dxc9000SendCommand(aux,"D300");
    Camera.D3 &= 0xDF;
  } else if (value == DXC9000_IRISMODE_FIX){
    Dxc9000SendCommand(aux,"D320");
    Camera.D3 |= 0x20;
  } 
  Dxc9000SendCommand(aux,"DC20");
}

/* ============================================================ */
/* GAIN (DXC9000_GAIN_AGC 뤤 DXC9000_GAIN_STEP) */
void Dxc9000Gain(int aux, int value){
  if (value == DXC9000_GAIN_AGC){
    Dxc9000SendCommand(aux,"D300");
    Camera.D3 &= 0x7F;
  } else if (value == DXC9000_GAIN_STEP){
    Dxc9000SendCommand(aux,"D380");
    Camera.D3 |= 0x80;
  } 
  Dxc9000SendCommand(aux,"DC80");
}

/* ============================================================ */
/* GAIN STEP (018dB) */
void Dxc9000GainStep(int aux, int value){
  char data[5];
  
  if (value < 0 || value > 18 ){
    fprintf(stderr,"Error: GAIN STEP is out of range.\n"); exit(1);
  }
  sprintf(data,"E2%02X",value);
  Dxc9000SendCommand(aux,data);
  Camera.gainstep = value;
}


/* ============================================================ */
/* DETAIL (DXC9000_OFF 뤤 DXC9000_ON) */
void Dxc9000Detail(int aux, int value){
  if (value == DXC9000_OFF){
    Dxc9000SendCommand(aux,"D100");
    Camera.D1 &= 0xFD;
  } else if (value == DXC9000_ON){
    Dxc9000SendCommand(aux,"D102");
    Camera.D1 |= 0x02;
  } else {
    fprintf(stderr,"Error: Gamma.\n"); exit(1);
  }
  Dxc9000SendCommand(aux,"DA02");
}

/* ============================================================ */
/* DETAIL-LEVEL (00xC6) */
void Dxc9000DetailLevel(int aux, int value){
  char data[5];
  
  if (value < 0 || value > 0xC6 ){
    fprintf(stderr,"Error: DETAIL LEVEL is out of range.\n"); exit(1);
  }
  sprintf(data,"M8%02X",value);
  Dxc9000SendCommand(aux,data);
  Camera.detaillevel = value;
}

/* ============================================================ */
/* Żҥå (DXC9000_SHUTTER_{STEP,VARIABLE,CCDIRIS,OFF) */
void Dxc9000Shutter(int aux, int value){
  if      (value == DXC9000_SHUTTER_STEP)     Dxc9000SendCommand(aux,"EA00");
  else if (value == DXC9000_SHUTTER_VARIABLE) Dxc9000SendCommand(aux,"EA01");
  else if (value == DXC9000_SHUTTER_CCDIRIS)  Dxc9000SendCommand(aux,"EA02");
  else if (value == DXC9000_SHUTTER_OFF)      Dxc9000SendCommand(aux,"EA03");
  else { fprintf(stderr,"Error: Shutter.\n"); exit(1); }

  Camera.shutter = value;
}

/* ============================================================ */
/* åԡɤ */
void Dxc9000ShutterSpeed(int aux, int value){
  /* EXT.TRIGGER ON/OFF ˤϰϤۤʤ */
  /* ǥեȤ OFFʤΤǡ0x00-0x1F Ȥʤ */
  char data[5];
  
  if (value < 0 || value > 0x1F ){
    fprintf(stderr,"Error: SHUTTER SPEED is out of range.\n"); exit(1);
  }
  sprintf(data,"EB%02X",value);
  Dxc9000SendCommand(aux,data);
  Camera.shutterspeed = value;
}

/* ============================================================ */
/* ۥ磻ȥХ󥹤 (DXC9000_WHITE_BALLANCE_{ATW,AWB,MANUAL) */
void Dxc9000WhiteBallance(int aux, int value){
  if      (value == DXC9000_WHITEBALLANCE_ATW) Dxc9000SendCommand(aux,"T200");
  else if (value == DXC9000_WHITEBALLANCE_AWB) Dxc9000SendCommand(aux,"T201");
  else if (value == DXC9000_WHITEBALLANCE_MANUAL)
    Dxc9000SendCommand(aux,"T210");
  else { fprintf(stderr,"Error: White Ballance.\n"); exit(1); }

  Camera.whiteballance = value;
}

/* ============================================================ */
/* ۥ磻ȥХ R Gain(0x000xFE) */
void Dxc9000RGain(int aux, int value){
  char data[5];
  
  if (value < 0 || value > 0xFE ){
    fprintf(stderr,"Error: R Gain is out of range.\n"); exit(1);
  }
  sprintf(data,"M1%02X",value);
  Dxc9000SendCommand(aux,data);

  Camera.rgain = value;
}

/* ============================================================ */
/* ۥ磻ȥХ B Gain(0x000xFE) */
void Dxc9000BGain(int aux, int value){
  char data[5];
  
  if (value < 0 || value > 0xFE ){
    fprintf(stderr,"Error: B Gain is out of range.\n"); exit(1);
  }
  sprintf(data,"M2%02X",value);
  Dxc9000SendCommand(aux,data);

  Camera.bgain = value;
}

/* ============================================================ */
/* SCAN-MODE (DXC9000_SCAN_{FS,NORMAL,VGA) */
void Dxc9000ScanMode(int aux, int value){
  if      (value == DXC9000_SCAN_FS)     Dxc9000SendCommand(aux,"SC00");
  else if (value == DXC9000_SCAN_NORMAL) Dxc9000SendCommand(aux,"SC01");
  else if (value == DXC9000_SCAN_VGA)    Dxc9000SendCommand(aux,"SC02");
  else { fprintf(stderr,"Error: Scan Mode.\n"); exit(1); }

  Camera.scanmode = value;
}

/* ============================================================ */
/* EXT.CTRL(DXC9000_EXTCTRL_{GENLOCK,TRIGIN} */
void Dxc9000ExtCtrl(int aux,int value){
  if (value == DXC9000_EXTCTRL_GENLOCK ){
    Dxc9000SendCommand(aux,"D100");
    Camera.D1 &= 0xF7;
  } else if (value == DXC9000_EXTCTRL_TRIGIN ){
    Dxc9000SendCommand(aux,"D108");
    Camera.D1 |= 0x08;
  } else {
    fprintf(stderr,"Error: EXT.CTRL.\n"); exit(1);
  }
  Dxc9000SendCommand(aux,"DA08");
}

/* ============================================================ */
/* H PHASE Fine(00xC6) */
void Dxc9000HPhase(int aux,int value){
  char data[5];
  
  if (value < 0 || value > 0xC6 ){
    fprintf(stderr,"Error: H PHASE is out of range.\n"); exit(1);
  }
  sprintf(data,"M9%02X",value);
  Dxc9000SendCommand(aux,data);
  Camera.hphase = value;
}


/* ============================================================ */
/* SC PHASE Rough(0 or 180) */
void Dxc9000SCPhaseRough(int aux,int value){
  if (value == 0){
    Dxc9000SendCommand(aux,"D300");
    Camera.D3 &= 0xEF;
  } else if (value == 1){
    Dxc9000SendCommand(aux,"D310");
    Camera.D3 |= 0x10;
  } else {
    fprintf(stderr,"Error: SC PHASE RoughL.\n"); exit(1);
  }
  Dxc9000SendCommand(aux,"DC10");
}


/* ============================================================ */
/* SC PHASE Fine(00xC6) */
void Dxc9000SCPhaseFine(int aux,int value){
  char data[5];
  
  if (value < 0 || value > 0xC6 ){
    fprintf(stderr,"Error: SC PHASE Fine is out of range.\n"); exit(1);
  }
  sprintf(data,"MA%02X",value);
  Dxc9000SendCommand(aux,data);
  Camera.scphasefine = value;
}
/* ============================================================ */
/* Color Bars ɽ (DXC9000_{OFF/ON}) */
void Dxc9000ColorBars(int aux,int value){

  if        (value == DXC9000_OFF){
    Dxc9000SendCommand(aux,"BA00");
    Camera.colorbars = DXC9000_OFF;
  } else if (value == DXC9000_ON ){
    Dxc9000SendCommand(aux,"BA01");
    Camera.colorbars = DXC9000_ON;
  }
  else { fprintf(stderr,"Error: Color Bars.\n"); exit(1); }
}
