#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "define.h"
#include "dxc9000.h"

static CommandType parse(char *command);

void writeStatus(FILE *p){
  time_t t;

  t = time((time_t *)NULL);

  /*--< $B%l%s%:$K4X$9$k%Q%i%a!<%?(B >--*/
  fprintf(p, "##### START %s", ctime(&t));
  fprintf(p, "%-15s%d\n",commandString[ZOOM ],Camera.zoom);
  fprintf(p, "%-15s%d\n",commandString[FOCUS],Camera.focus);
  if (Camera.D3 & 0x20) fprintf(p, "%-15sFIX\n", commandString[IRISMODE]);
  else                  fprintf(p, "%-15sAUTO\n",commandString[IRISMODE]);
  fprintf(p, "%-15s%d\n",commandString[IRIS],Camera.iris);

  /*--< $B?'$dL@$k$5$J$I$K4X$9$k%Q%i%a!<%?(B >--*/
  if (Camera.D1 & 0x80) fprintf(p, "%-15s5600K\n",commandString[CTEMP]);
  else                  fprintf(p, "%-15s3200K\n",commandString[CTEMP]);
  if (Camera.D1 & 0x40) fprintf(p, "%-15sON\n" ,  commandString[GAMMA]);
  else                  fprintf(p, "%-15sOFF\n",  commandString[GAMMA]);
  if (Camera.D3 & 0x80) fprintf(p, "%-15sSTEP\n", commandString[GAIN]);
  else                  fprintf(p, "%-15sAGC\n",  commandString[GAIN]);
  fprintf(p, "%-15s%d\n",commandString[GAINSTEP], Camera.gainstep);
  if (Camera.D1 & 0x02) fprintf(p, "%-15sON\n",   commandString[DETAIL]);
  else                  fprintf(p, "%-15sOFF\n",  commandString[DETAIL]);
  fprintf(p, "%-15s%d\n",commandString[DETAILLEVEL], Camera.detaillevel);

  /*--< $B%7%c%C%?!<%9%T!<%I$K4X$9$k%Q%i%a!<%?(B >--*/
  fprintf(p, "%-15s",commandString[SHUTTER]);
  switch(Camera.shutter){
  case DXC9000_SHUTTER_STEP:     fprintf(p, "STEP\n");     break;
  case DXC9000_SHUTTER_VARIABLE: fprintf(p, "VARIABLE\n"); break;
  case DXC9000_SHUTTER_CCDIRIS:  fprintf(p, "CCDIRIS\n");  break;
  case DXC9000_SHUTTER_OFF:      fprintf(p, "OFF\n");      break;
  default: fprintf(stderr,"Format error (SHUTTER)\n"); exit(1);
  }
  fprintf(p, "%-15s%s\n",commandString[SHUTTERSPEED],
	  shutterString[Camera.shutterspeed]);

  /*--< $B%[%o%$%H%P%i%s%9$K4X$9$k%Q%i%a!<%?(B >--*/
  fprintf(p, "%-15s",commandString[WHITEBALLANCE]);
  switch(Camera.whiteballance){
  case DXC9000_WHITEBALLANCE_ATW:    fprintf(p, "ATW\n");    break;
  case DXC9000_WHITEBALLANCE_AWB:    fprintf(p, "AWB\n");    break;
  case DXC9000_WHITEBALLANCE_MANUAL: fprintf(p, "MANUAL\n"); break;
  default: fprintf(stderr,"Format error (WHITEBALLANCE)\n"); exit(1);
  }
  fprintf(p, "%-15s%d\n",commandString[RGAIN],Camera.rgain);
  fprintf(p, "%-15s%d\n",commandString[BGAIN],Camera.bgain);


  /*--< $B%9%-%c%s%b!<%I$K4X$9$k%Q%i%a!<%?(B >--*/
  fprintf(p, "%-15s",commandString[SCANMODE]);
  switch(Camera.scanmode){
  case DXC9000_SCAN_FS:     fprintf(p, "FS\n");     break;
  case DXC9000_SCAN_NORMAL: fprintf(p, "NORMAL\n"); break;
  case DXC9000_SCAN_VGA:    fprintf(p, "VGA\n");    break;
  default: fprintf(stderr,"Format error (SCANMODE)\n"); exit(1);
  }

  /*--< $B30ItF14|$K4X$9$k%Q%i%a!<%?(B >--*/
  if (Camera.D1 & 0x08) fprintf(p, "%-15sTRIG-IN\n",commandString[EXTCTRL]);
  else                  fprintf(p, "%-15sGENLOCK\n",commandString[EXTCTRL]);
  fprintf(p, "%-15s%d\n",commandString[HPHASE],     Camera.hphase);  
  if (Camera.D3 & 0x10) fprintf(p, "%-15s180\n", commandString[SCPHASEROUGH]);
  else                  fprintf(p, "%-15s0\n",   commandString[SCPHASEROUGH]);
  fprintf(p, "%-15s%d\n",commandString[SCPHASEFINE],Camera.scphasefine);  

  /*--< Color Bars >--*/
  if (Camera.colorbars == DXC9000_OFF)
    fprintf(p, "%-15sOFF\n",commandString[COLORBARS]);
  else 
    fprintf(p, "%-15sON\n" ,commandString[COLORBARS]);

  /*--< $BK\%D!<%k$G@_Dj$G$-$J$$$=$NB>$N%Q%i%a!<%?(B >--*/
  if (Camera.D1 & 0x20) fprintf(p, "%-15sON\n" ,    commandString[GSYNC]);
  else                  fprintf(p, "%-15sOFF\n",    commandString[GSYNC]);
  if (Camera.D1 & 0x10) fprintf(p, "%-15sVBS\n",    commandString[DSUBVIDEO]);
  else                  fprintf(p, "%-15sY/C\n",    commandString[DSUBVIDEO]);
  if (Camera.D1 & 0x04) fprintf(p, "%-15sEXT.CTRL\n",commandString[FREEZE]);
  else                  fprintf(p, "%-15sINT.CTRL\n",commandString[FREEZE]);
  if (Camera.D1 & 0x01) fprintf(p, "%-15sAVERAGE\n",commandString[AEDETECTION]);
  else                  fprintf(p, "%-15sPEAK\n",   commandString[AEDETECTION]);
  if (Camera.D2 & 0x80) fprintf(p, "%-15sON\n",  commandString[EXTTRIGGER]);
  else                  fprintf(p, "%-15sOFF\n", commandString[EXTTRIGGER]);
  if (Camera.D2 & 0x20) fprintf(p, "%-15sF/F\n", commandString[FREEZEMODE]);
  else                  fprintf(p, "%-15sF/S\n", commandString[FREEZEMODE]);
  if (Camera.D2 & 0x08) fprintf(p, "%-15sON\n",  commandString[PROTECT]);
  else                  fprintf(p, "%-15sOFF\n", commandString[PROTECT]);
  if (Camera.D2 & 0x04) fprintf(p, "%-15sDOWN\n",commandString[TRIGPULSE]);
  else                  fprintf(p, "%-15sUP\n",  commandString[TRIGPULSE]);

  // USER-PRESET$B$O!"$3$N%D!<%k$G07$&$3$H<+BN$,$*$+$7$$(B
  //  if (Camera.D3 & 0x40) fprintf(p, "USER-PRESET    B\n");
  //  else                  fprintf(p, "USER-PRESET    A\n");

  fprintf(p, "##### END %s", ctime(&t));
}

void readStatus(FILE *fp){
  int i,find;
  CommandType c;
  char line[LINEMAX], command[LINEMAX], arg[LINEMAX];
  
  while (!feof(fp)){
    fgets(line,LINEMAX,fp);
    if (sscanf(line,"%s",command) != 1) continue;
    if (command[0]=='#') continue;
    if (sscanf(line,"%s%s",command,arg) != 2) continue;
    // printf("Com:%s, Arg:%s\n",command, arg);

    c = parse(command);
    if (c == -1 ) {
      fprintf(stderr,"Error: Unknown command (%s)\n",command);
      exit(1);
    }
      
    
    switch (c){
    case ZOOM:  Camera.zoom  = atoi(arg); break;
    case FOCUS: Camera.focus = atoi(arg); break;
    case IRIS:  Camera.iris  = atoi(arg); break;

    case IRISMODE:
      if      (strcmp(arg,"AUTO" )==0) Camera.D3 &= 0xDF;
      else if (strcmp(arg,"FIX"  )==0) Camera.D3 |= 0x20;
      else { fprintf(stderr,"Format error (IRIS-MODE)\n"); exit(1);}
      break;

    case CTEMP:
      if      (strcmp(arg,"3200K" )==0) Camera.D1 &= 0x7F;
      else if (strcmp(arg,"5600K" )==0) Camera.D1 |= 0x80;
      else { fprintf(stderr,"Format error (C.TEMP)\n"); exit(1);}
      break;

    case GAMMA:
      if      (strcmp(arg,"OFF" )==0) Camera.D1 &= 0xBF;
      else if (strcmp(arg,"ON"  )==0) Camera.D1 |= 0x40;
      else { fprintf(stderr,"Format error (GAMMA)\n"); exit(1);}
      break;

    case GAIN:
      if      (strcmp(arg,"AGC"  )==0) Camera.D3 &= 0x7F;
      else if (strcmp(arg,"STEP" )==0) Camera.D3 |= 0x80;
      else { fprintf(stderr,"Format error (GAIN)\n"); exit(1);}
      break;

    case GAINSTEP: Camera.gainstep = atoi(arg); break;

    case DETAIL:
      if      (strcmp(arg,"OFF" )==0) Camera.D1 &= 0xFD;
      else if (strcmp(arg,"ON"  )==0) Camera.D1 |= 0x02;
      else { fprintf(stderr,"Format error (DETAIL)\n"); exit(1);}
      break;

    case DETAILLEVEL: Camera.detaillevel = atoi(arg); break;

    case SHUTTER:      
      if      (strcmp(arg,"STEP"    )==0)
	Camera.shutter = DXC9000_SHUTTER_STEP;
      else if (strcmp(arg,"VARIABLE")==0)
	Camera.shutter = DXC9000_SHUTTER_VARIABLE;
      else if (strcmp(arg,"CCDIRIS" )==0)
	Camera.shutter = DXC9000_SHUTTER_CCDIRIS;
      else if (strcmp(arg,"OFF"     )==0)
	Camera.shutter = DXC9000_SHUTTER_OFF;
      else { fprintf(stderr,"Format error (SHUTTER)\n"); exit(1);}
      break;

    case SHUTTERSPEED:
      find = -1;
      for (i=0;i<32;i++){
	if (strcmp(shutterString[i],arg) == 0 ){
	  find = i;
	  break;
	}
      }
      if (find != -1) Camera.shutterspeed = i;
      else  { fprintf(stderr,"Format error (SHUTTERSPEED)\n"); exit(1);}
      break;

    case WHITEBALLANCE:    
      if      (strcmp(arg,"ATW" )==0)
	Camera.whiteballance = DXC9000_WHITEBALLANCE_ATW;
      else if (strcmp(arg,"AWB" )==0)
	Camera.whiteballance = DXC9000_WHITEBALLANCE_AWB;
      else if (strcmp(arg,"MANUAL" )==0)
	Camera.whiteballance = DXC9000_WHITEBALLANCE_MANUAL;
      else { fprintf(stderr,"Format error (WHITE BALLANCE)\n"); exit(1);}
      break;

    case RGAIN: Camera.rgain = atoi(arg); break;
    case BGAIN: Camera.bgain = atoi(arg); break;

    case SCANMODE:
      if      (strcmp(arg,"FS"    )==0) Camera.scanmode = DXC9000_SCAN_FS;
      else if (strcmp(arg,"NORMAL")==0) Camera.scanmode = DXC9000_SCAN_NORMAL;
      else if (strcmp(arg,"VGA"   )==0) Camera.scanmode = DXC9000_SCAN_VGA;
      break;

    case EXTCTRL:
      if      (strcmp(arg,"TRIG-IN")==0) Camera.D1 |= 0x08;
      else if (strcmp(arg,"GENLOCK")==0) Camera.D1 &= 0xF7;
      else { fprintf(stderr,"Format error (EXT.CTRL)\n"); exit(1);}
      break;

    case HPHASE: Camera.hphase = atoi(arg); break;

    case SCPHASEROUGH: 
      if      (strcmp(arg,"0")  ==0) Camera.D3 &= 0xEF;
      else if (strcmp(arg,"180")==0) Camera.D3 |= 0x10;
      else { fprintf(stderr,"Format error (EXT.CTRL)\n"); exit(1);}
      break;

    case SCPHASEFINE: Camera.scphasefine = atoi(arg); break;

    case COLORBARS:
      if      (strcmp(arg,"OFF")==0) Camera.colorbars = DXC9000_OFF;
      else if (strcmp(arg,"ON" )==0) Camera.colorbars = DXC9000_ON;
      else { fprintf(stderr,"Format error (COLORBARS)\n"); exit(1);}
      break;

    /*--< $B0J2<$O%5%]!<%H$7$J$$%3%^%s%I(B >--*/

    case GSYNC:
    case DSUBVIDEO:
    case FREEZE:
    case AEDETECTION:
    case EXTTRIGGER:
    case FREEZEMODE:
    case PROTECT:
    case TRIGPULSE:
      break;

    default:
      fprintf(stderr,"Error: not supported command\n");
      exit(1);
    }
  }
}


static CommandType parse(char *command){
  int i;
  
  for (i=0; i<COMMANDNUM; i++){
    if (strcmp(command, commandString[i])==0) return i;
  }
  fprintf(stderr,"Format error (unknown parameter name '%s')\n",command);
  return -1;
}



