#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "define.h"
#include "serial.h"
#include "dxc9000.h"

typedef
enum { ShutterOFF,ShutterSTEP,ShutterCCDIRIS,
       C3200K, C5600K,
       GammaOFF, GammaON,
       IrisModeAuto, IrisModeFix,
       ExtCtrlTrigIn, ExtCtrlGenlock,
       ColorbarOFF,ColorbarON
       }
RadioEvent;

void writeStatus(FILE *p);

static void cb_quit(GtkWidget *widget, gpointer *data);
static void cb_print(GtkWidget *widget, gpointer *data);
static void cb_save(GtkWidget *widget, gpointer *data);
static void cb_zoom(GtkAdjustment *adj, gpointer *data);
static void cb_focus(GtkAdjustment *adj, gpointer *data);
static void cb_iris(GtkAdjustment *adj, gpointer *data);
static void cb_select(GtkWidget *widget, gpointer data);
static void cb_shutter(GtkWidget *widget, gpointer data);

#define YINT1 30 // $B%9%/%m!<%k%P!<$N=DJ}8~$N%$%s%?!<%P%k(B
#define YINT2 25 // $B$=$NB>$N%"%$%F%`$N=DJ}8~$N%$%s%?!<%P%k(B
#define XINT  80 // $B%"%$%F%`L>$N<!(B($BBh#2NsL\(B)$B$N2#J}8~$N%$%s%?!<%P%k(B

#define HOSTNAMEMAX 100

GtkWidget *combo;

/*--< GUI$B%9%/%j!<%s$N%;%C%H%"%C%W(B >--*/
void guiSetup(void){
  int i;
  int y=10;
  char hostname[HOSTNAMEMAX];

  GtkWidget *window, *fixed;
  GtkWidget *button_quit,*button_save,*button_print, *button_select;
  GtkWidget *hscale_zoom, *hscale_iris, *hscale_focus;
  GtkWidget *label;
  GtkWidget *radio;
  GtkObject *adj_zoom, *adj_iris, *adj_focus;
  GList *glist=NULL;

  gethostname(hostname, HOSTNAMEMAX);

  /* $B%H%C%W%l%Y%k$N%&%#%s%I%&$r:n$k(B */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW(window), hostname);

  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		     GTK_SIGNAL_FUNC(cb_quit), NULL);

  /* $B%&%#%s%I%&$N%\!<%@I}$r@_Dj(B */
  gtk_container_set_border_width (GTK_CONTAINER (window), 10);

  /* Fixed container $B$r:n@.(B */
  fixed = gtk_fixed_new();
  gtk_container_add(GTK_CONTAINER(window), fixed);
  gtk_widget_show(fixed);

  /* ---------------------------------------------------------------------- */
  /* Zoom$B%3%s%H%m!<%k%P!<(B */
  label = gtk_label_new("ZOOM:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);

  adj_zoom = gtk_adjustment_new(Camera.zoom, 0, 255, 1, 1, 0);
  gtk_signal_connect(GTK_OBJECT(adj_zoom), "value_changed",
		     GTK_SIGNAL_FUNC(cb_zoom),NULL);
  hscale_zoom = gtk_hscale_new(GTK_ADJUSTMENT(adj_zoom));
  gtk_widget_set_usize(hscale_zoom,256,30);
  gtk_scale_set_digits(GTK_SCALE(hscale_zoom),0); /* $B>.?tE@0J2<$OI=<($7$J$$(B */
  gtk_widget_show(hscale_zoom);
  gtk_fixed_put (GTK_FIXED(fixed), hscale_zoom, XINT, y-10);  
  y += YINT1;

  /* ---------------------------------------------------------------------- */
  /* Focus$B%3%s%H%m!<%k%P!<(B */
  label = gtk_label_new("FOCUS:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);  

  adj_focus = gtk_adjustment_new(Camera.focus, 0, 255, 1, 1, 0);
  gtk_signal_connect(GTK_OBJECT(adj_focus), "value_changed",
		     GTK_SIGNAL_FUNC(cb_focus),NULL);
  hscale_focus = gtk_hscale_new(GTK_ADJUSTMENT(adj_focus));
  gtk_widget_set_usize(hscale_focus,256,30);
  gtk_scale_set_digits(GTK_SCALE(hscale_focus),0); /* $B>.?tE@0J2<$OI=<($7$J$$(B */
  gtk_widget_show(hscale_focus);
  gtk_fixed_put (GTK_FIXED(fixed), hscale_focus, XINT, y-10);
  y += YINT1;

  /* ---------------------------------------------------------------------- */
  /* Iris$B%3%s%H%m!<%k%P!<(B */
  label = gtk_label_new("IRIS:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);  

  adj_iris = gtk_adjustment_new(Camera.iris, 0, 255, 1, 1, 0);
  gtk_signal_connect(GTK_OBJECT(adj_iris), "value_changed",
		     GTK_SIGNAL_FUNC(cb_iris),NULL);
  hscale_iris = gtk_hscale_new(GTK_ADJUSTMENT(adj_iris));
  gtk_widget_set_usize(hscale_iris,256,30);
  gtk_scale_set_digits(GTK_SCALE(hscale_iris),0); /* $B>.?tE@0J2<$OI=<($7$J$$(B */
  gtk_widget_show(hscale_iris);
  gtk_fixed_put (GTK_FIXED(fixed), hscale_iris, XINT, y-10);  
  y += YINT1;

  /* ---------------------------------------------------------------------- */
  /* $BEE;R%7%c%C%?!<(B $B%i%8%*%\%?%s(B */
  label = gtk_label_new("SHUTTER:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);  

  radio = gtk_radio_button_new_with_label( NULL, "OFF");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT, y-5);
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)ShutterOFF);
  if (Camera.shutter == DXC9000_SHUTTER_OFF)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  radio = gtk_radio_button_new_with_label(
  gtk_radio_button_group (GTK_RADIO_BUTTON (radio)), "STEP");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT+70, y-5);
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)ShutterSTEP);
  if (Camera.shutter == DXC9000_SHUTTER_STEP)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  radio = gtk_radio_button_new_with_label(
  gtk_radio_button_group (GTK_RADIO_BUTTON (radio)), "CCD IRIS");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT+140, y-5);
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)ShutterCCDIRIS);
  if (Camera.shutter == DXC9000_SHUTTER_CCDIRIS)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  y += YINT2;

  /* ---------------------------------------------------------------------- */
  /* $BEE;R%7%c%C%?!<(B $B%3%s%\%\%C%/%9(B */
  label = gtk_label_new("SHUTTER SPEED:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);

  for (i=0;i<32;i++) glist = g_list_append(glist, shutterString[i]);
  combo = gtk_combo_new();
  gtk_combo_set_popdown_strings( GTK_COMBO(combo), glist);
  gtk_widget_set_usize(combo,150,22);
  gtk_widget_show(combo);
  gtk_fixed_put (GTK_FIXED(fixed), combo, XINT+50, y-5);  
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "activate",
		     GTK_SIGNAL_FUNC (cb_shutter), NULL);
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), 
		     shutterString[Camera.shutterspeed]);


  /* Select Button */
  button_select = gtk_button_new_with_label(" Select ");
  gtk_signal_connect(GTK_OBJECT(button_select), "clicked",
			   GTK_SIGNAL_FUNC(cb_shutter), NULL);
  gtk_widget_show(button_select);
  gtk_fixed_put(GTK_FIXED(fixed), button_select, XINT+210, y-5);

  y += YINT2;

  /* ---------------------------------------------------------------------- */
  /* C.TEMP $B%i%8%*%\%?%s(B */
  label = gtk_label_new("C.TEMP:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);  

  radio = gtk_radio_button_new_with_label( NULL, "3200K");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT, y-5);
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)C3200K);
  if ((Camera.D1 & 0x80) == 0)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  radio = gtk_radio_button_new_with_label(
  gtk_radio_button_group (GTK_RADIO_BUTTON (radio)), "5600K");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT+70, y-5);  
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)C5600K);
  if ((Camera.D1 & 0x80) != 0)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  y += YINT2;

  /* ---------------------------------------------------------------------- */
  /* GAMMA $B%i%8%*%\%?%s(B */
  label = gtk_label_new("GAMMA:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);  

  radio = gtk_radio_button_new_with_label( NULL, "OFF");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT, y-5);
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)GammaOFF);
  if ((Camera.D1 & 0x40) == 0)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  radio = gtk_radio_button_new_with_label(
  gtk_radio_button_group (GTK_RADIO_BUTTON (radio)), "ON");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT+70, y-5);  
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)GammaON);
  if ((Camera.D1 & 0x40) != 0)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  y += YINT2;

  /* ---------------------------------------------------------------------- */
  /* IRIS MODE $B%i%8%*%\%?%s(B */
  label = gtk_label_new("IRIS MODE:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);  

  radio = gtk_radio_button_new_with_label( NULL, "AUTO");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT, y-5);
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)IrisModeAuto);
  if ((Camera.D3 & 0x20) == 0)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  radio = gtk_radio_button_new_with_label(
  gtk_radio_button_group (GTK_RADIO_BUTTON (radio)), "FIX");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT+70, y-5);  
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)IrisModeFix);
  if ((Camera.D3 & 0x20) != 0)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  y += YINT2;

  /* ---------------------------------------------------------------------- */
  /* EXT.CTRL$B%i%8%*%\%?%s(B */
  label = gtk_label_new("EXT.CTRL:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);  

  radio = gtk_radio_button_new_with_label( NULL, "TRIG IN");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT, y-5);
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)ExtCtrlTrigIn);
  

  if ((Camera.D1 & 0x08) != 1)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  radio = gtk_radio_button_new_with_label(
  gtk_radio_button_group (GTK_RADIO_BUTTON (radio)), "GENLOCK");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT+70, y-5);  
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)ExtCtrlGenlock);
  if ((Camera.D1 & 0x08) == 0)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  y += YINT2;

  /* ---------------------------------------------------------------------- */
  /*$B%+%i!<%P!<%i%8%*%\%?%s(B */
  label = gtk_label_new("COLOR BAR:");
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0, y);  

  radio = gtk_radio_button_new_with_label( NULL, "OFF");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT, y-5);
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)ColorbarOFF);
  
  if (Camera.colorbars == DXC9000_OFF)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  radio = gtk_radio_button_new_with_label(
  gtk_radio_button_group (GTK_RADIO_BUTTON (radio)), "ON");
  gtk_widget_show(radio);
  gtk_fixed_put (GTK_FIXED(fixed), radio, XINT+70, y-5);  
  gtk_signal_connect(GTK_OBJECT (radio), "clicked",
		     GTK_SIGNAL_FUNC(cb_select), (gpointer)ColorbarON);
  if (Camera.colorbars == DXC9000_ON)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);

  y += YINT2;

  /* ---------------------------------------------------------------------- */
  /* $B%P!<%8%g%sI=5-(B */
  label = gtk_label_new(VERSION);
  gtk_widget_show(label);
  gtk_fixed_put (GTK_FIXED(fixed), label, 0,y+5);

  /* ---------------------------------------------------------------------- */
  /* Print Button */
  button_print = gtk_button_new_with_label(" Print ");
  gtk_signal_connect(GTK_OBJECT(button_print), "clicked",
			   GTK_SIGNAL_FUNC(cb_print), NULL);
  gtk_widget_show(button_print);
  gtk_fixed_put (GTK_FIXED(fixed), button_print, 200,y);

  /* ---------------------------------------------------------------------- */
  /* Save Button */
  button_save = gtk_button_new_with_label(" Save ");
  gtk_signal_connect(GTK_OBJECT(button_save), "clicked",
			   GTK_SIGNAL_FUNC(cb_save), NULL);
  gtk_widget_show(button_save);
  gtk_fixed_put (GTK_FIXED(fixed), button_save, 250,y);

  /* ---------------------------------------------------------------------- */
  /* Quit Button */
  button_quit = gtk_button_new_with_label(" Quit ");
  gtk_signal_connect(GTK_OBJECT(button_quit), "clicked",
			   GTK_SIGNAL_FUNC(cb_quit), NULL);
  gtk_widget_show(button_quit);
  gtk_fixed_put (GTK_FIXED(fixed), button_quit, 300,y);  

  /* ---------------------------------------------------------------------- */
  /* Window$B$NI=<((B */
  gtk_widget_show(window);

}

/*--< Quit$B%3!<%k%P%C%/(B >--*/
static void cb_quit(GtkWidget *widget, gpointer *data){
  SerialClose(port);
  gtk_main_quit();
}

/*--< Save$B%3!<%k%P%C%/(B >--*/
static void cb_save(GtkWidget *widget, gpointer *data){
  FILE *fp;
  if (OPT_OUTPUT == OPT_TRUE){
    fp = fopen(OPT_OUTPUT_NAME,"w");
    writeStatus(fp);
    fclose(fp);
    writeStatus(stdout);
  } else {
    fprintf(stderr,"Error: Filename is not specified.\n");
    fprintf(stderr,"If you want to save parameters, use -o option.\n");
  }
}

/*--< Print$B%3!<%k%P%C%/(B >--*/
static void cb_print(GtkWidget *widget, gpointer *data){
  writeStatus(stdout);
}

/*--< Zoom >--*/
static void cb_zoom(GtkAdjustment *adj, gpointer *data){
  Dxc9000Zoom(port, (int)adj->value);
}
/*--< Focus >--*/
static void cb_focus(GtkAdjustment *adj, gpointer *data){
  Dxc9000Focus(port, (int)adj->value);
}
/*--< Iris >--*/
static void cb_iris(GtkAdjustment *adj, gpointer *data){
  Dxc9000Iris(port, (int)adj->value);
}

/* Radio Button */
static void cb_select (GtkWidget *widget, gpointer data){
  //  if (!GTK_TOGGLE_BUTTON (widget)->active) return;

  switch ((int)data){
  case ShutterOFF:
    Dxc9000Shutter(port, DXC9000_SHUTTER_OFF);
    break;
  case ShutterSTEP:
    Dxc9000Shutter(port, DXC9000_SHUTTER_STEP);
    break;
  case ShutterCCDIRIS:
    Dxc9000Shutter(port, DXC9000_SHUTTER_CCDIRIS);
    break;
  case C3200K:
    Dxc9000ColorTemperture(port, DXC9000_CTEMP_3200K);
    break;
  case C5600K:
    Dxc9000ColorTemperture(port, DXC9000_CTEMP_5600K);
    break;
  case GammaOFF:
    Dxc9000Gamma(port, DXC9000_OFF);
    break;    
  case GammaON:
    Dxc9000Gamma(port, DXC9000_ON);
    break;
  case IrisModeAuto:
    Dxc9000IrisMode(port, DXC9000_IRISMODE_AUTO);
    break;
  case IrisModeFix:
    Dxc9000IrisMode(port, DXC9000_IRISMODE_FIX);
    break;
  case ExtCtrlTrigIn:
    Dxc9000ExtCtrl(port, DXC9000_EXTCTRL_TRIGIN);
    break; 
  case ExtCtrlGenlock:
    Dxc9000ExtCtrl(port, DXC9000_EXTCTRL_GENLOCK);
    break;
  case ColorbarOFF:
    Dxc9000ColorBars(port,DXC9000_OFF);
    break;
  case ColorbarON:
    Dxc9000ColorBars(port,DXC9000_ON);
    break;
  default:
    fprintf(stderr, "ERROR: Unknown event in GUI process\n");
  }
}

/*--< Shutter >--*/
static void cb_shutter(GtkWidget *widget, gpointer data){
  int i;
  //  printf("Shutter:%s\n",
  //	 (char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry)));
  for (i=0;i<32;i++){
    if (strcmp(shutterString[i],(char *)gtk_entry_get_text(
	   GTK_ENTRY(GTK_COMBO(combo)->entry)))	== 0 ){
      Dxc9000ShutterSpeed(port, i);
      Camera.shutterspeed = i;
      break;
    }
  }
}
