#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <gtk/gtk.h>
#include "define.h"
#include "serial.h"
#include "dxc9000.h"

void guiSetup(void);
void readStatus(FILE *fp);

static void myinit(void);
static void cameraState(void);
static void cameraInit(void);
static void usage(char *s);


int main(int argc, char *argv[]){

  int           ch;
  extern char   *optarg;
  extern int    optind;

  /*--< $B%*%W%7%g%s4X78$K4X$9$k=i4|2=(B >--*/
  myinit();

  if (OPT_COMMAND == OPT_FALSE) gtk_init(&argc, &argv);

  /*--< $B%*%W%7%g%s2r@O(B >--*/
  while ((ch = getopt(argc, argv, "ci:o:d:D")) != -1){
    switch (ch){
    case 'i':
      OPT_INPUT = OPT_TRUE;
      strncpy(OPT_INPUT_NAME, optarg, FILEMAX);
      break;
    case 'o':
      OPT_OUTPUT = OPT_TRUE;
      strncpy(OPT_OUTPUT_NAME, optarg, FILEMAX);
      break;
    case 'd':
      PORTNAME = optarg;
      break;
    case 'c':
      OPT_COMMAND = OPT_TRUE;
      break;
    case 'D':
      OPT_DEBUG = OPT_TRUE;
      break;
    case 'h':
    default:
      usage(argv[0]);
    }
  }

  /*--< $B%7%j%"%k%]!<%H$r3+$$$F30It%3%^%s%I%b!<%I$K$9$k(B >--*/
  if (OPT_DEBUG == OPT_FALSE){
    port = SerialOpen(PORTNAME, 9600);
    Dxc9000SetUp(port);
  }
  
  /*--< $B%+%a%i$N=i4|@_Dj>uBV$r7h$a$k(B >--*/
  cameraState();

  /*--< $B%+%a%i$r=i4|>uBV$K%;%C%H$9$k(B >--*/
  cameraInit();
  
  /*--< GUI$B%b!<%I(B($BHs%3%^%s%I%b!<%I(B)$B$J$i(B GUI$B$r;H$&(B >--*/
  if (OPT_COMMAND == OPT_FALSE){
    guiSetup();
    gtk_main();
  }
  return 0;
}


static void myinit(void){
  OPT_INPUT   = OPT_FALSE;
  OPT_OUTPUT  = OPT_FALSE;
  OPT_COMMAND = OPT_FALSE;
  OPT_DEBUG   = OPT_FALSE;
  PORTNAME    = COMPORT1;
}


/*--< $B%+%a%i$N=i4|>uBV$N%;%C%H(B >--*/
static void cameraState(void){
  FILE *fp;

  /* $B%G%U%)%k%HCM(B */
  Camera.zoom  = 128;
  Camera.focus = 128;
  Camera.iris  = 128;
  Camera.gainstep = 0;
  Camera.detaillevel = 0x63;
  Camera.shutter = DXC9000_SHUTTER_OFF;
  Camera.shutterspeed = 0x17;
  Camera.whiteballance = DXC9000_WHITEBALLANCE_MANUAL;
  Camera.rgain = 0x7F;
  Camera.bgain = 0x7F;
  Camera.scanmode = DXC9000_SCAN_FS;
  Camera.hphase = 0x63;
  Camera.scphasefine = 0x63;
  Camera.colorbars = DXC9000_OFF;
  Camera.D1 = 0xEB;
  Camera.D2 = 0x04;
  Camera.D3 = 0x80;

  if (OPT_INPUT == OPT_TRUE){
    if ((fp = fopen(OPT_INPUT_NAME, "r")) == NULL){
      fprintf(stderr,"File open error (%s)\n",OPT_INPUT_NAME);
      exit(1);
    }
    readStatus(fp);
    fclose(fp);
  }
}

static void cameraInit(void){
  Dxc9000Zoom (port, Camera.zoom);
  Dxc9000Focus(port, Camera.focus);
  Dxc9000Iris (port, Camera.iris);
  Dxc9000GainStep(port, Camera.gainstep);
  Dxc9000DetailLevel(port, Camera.detaillevel);
  Dxc9000Shutter(port, Camera.shutter);
  Dxc9000ShutterSpeed(port, Camera.shutterspeed);
  Dxc9000WhiteBallance(port, Camera.whiteballance);
  Dxc9000RGain(port, Camera.rgain);
  Dxc9000BGain(port, Camera.bgain);
  Dxc9000ScanMode(port, Camera.scanmode);
  Dxc9000HPhase(port, Camera.hphase);
  Dxc9000SCPhaseFine(port, Camera.scphasefine);
  Dxc9000ColorBars(port, Camera.colorbars);

  Dxc9000State(port, 1, Camera.D1, 0xFF);
  Dxc9000State(port, 2, Camera.D2, 0xAC);
  Dxc9000State(port, 3, Camera.D3, 0xB0);
}  

static void usage(char *s){

  fprintf(stderr,"#####[ %s ]#####\n",VERSION);
  fprintf(stderr,"Usage:   %s [options]\n",s);
  fprintf(stderr,"Options: -o# Set output file name\n");
  fprintf(stderr,"         -i# Read initial parameters file\n");
  fprintf(stderr,"         -d# Device name for RS-232C(default:/dev/cua0)\n");
  fprintf(stderr,"         -c  Command mode (not GUI)\n");
  fprintf(stderr,"         -D  Debug mode. No communication with DXC9000.\n");
  exit(1);
}
